function __async(f) {
	return function () {
		try {
			return Promise.resolve(f.apply(this, arguments));
		} catch (e) {
			return Promise.reject(e);
		}
	};
}

function __await(value, then) {
	return (value && value.then ? value : Promise.resolve(value)).then(then);
}

function __forTo(array, body) {
	var i = 0;
	return __for(function () {
		return i < array.length;
	}, function () {
		i++;
	}, function () {
		return body(i);
	});
}

function __forIn(target, body, check) {
	var keys = [],
	    i = 0;

	for (var key in target) {
		keys.push(key);
	}

	return __for(check ? function () {
		return i < keys.length && check();
	} : function () {
		return i < keys.length;
	}, function () {
		i++;
	}, function () {
		return body(keys[i]);
	});
}

function __switch(discriminant, cases) {
	return new Promise(function (resolve, reject) {
		var i = -1;
		var defaultIndex = -1;

		function nextCase() {
			if (++i === cases.length) {
				if (defaultIndex !== -1) {
					i = defaultIndex;
					dispatchCaseBody();
				} else {
					resolve();
				}
			} else {
				var test = cases[i][0];

				if (test) {
					__try(test).then(checkCaseTest, reject);
				} else {
					defaultIndex = i;
					nextCase();
				}
			}
		}

		function checkCaseTest(test) {
			if (test !== discriminant) {
				nextCase();
			} else {
				dispatchCaseBody();
			}
		}

		function dispatchCaseBody() {
			for (;;) {
				var body = cases[i][1];

				if (body) {
					return __try(body).then(checkFallthrough, reject);
				} else if (++i === cases.length) {
					return resolve();
				}
			}
		}

		function checkFallthrough(result) {
			var fallthroughCheck = cases[i][2];

			if (!fallthroughCheck || fallthroughCheck()) {
				resolve(result);
			} else if (++i === cases.length) {
				resolve();
			} else {
				dispatchCaseBody();
			}
		}

		nextCase();
	});
}

function __for(test, update, body) {
	return new Promise(function (resolve, reject) {
		var result;
		cycle();

		function cycle() {
			__try(test).then(checkTestResult, reject);
		}

		function stashAndUpdate(value) {
			result = value;
			return update && update();
		}

		function checkTestResult(shouldContinue) {
			if (shouldContinue) {
				__try(body).then(stashAndUpdate).then(cycle, reject);
			} else {
				resolve(result);
			}
		}
	});
}

function __do(body, test) {
	return new Promise(function (resolve, reject) {
		cycle();

		function cycle() {
			return __try(body).then(checkTestResult, reject);
		}

		function checkTestResult(value) {
			__try(test).then(function (shouldContinue) {
				if (shouldContinue) {
					cycle();
				} else {
					resolve(value);
				}
			}, reject);
		}
	});
}

function __try(body) {
	return new Promise(function (resolve) {
		resolve(body());
	});
}

function __finally(promise, finalizer) {
	return promise.then(finalizer.bind(null, false), finalizer.bind(null, true));
}

function __empty() {}

var simpleAsync = __async(function () {
	return true;
}),
    awaitPassthrough = __async(function () {
	return fetch("http://www.google.com/");
});

function notAsync() {
	return false;
}

var callChain = __async(function () {
	return __await(bar(), function (_bar) {
		var _ref = _bar,
		    _foo = foo;
		return __await(_foo(_ref, _baz), function (_foo2) {
			return __await(baz(), function (_baz) {
				return _foo2;
			});
		});
	});
});

var argumentStash = __async(function (fn) {
	var _foo3 = foo();

	return __await(bar(), function (_bar2) {
		return fn(1, _foo3, _bar2, baz());
	});
});

var arrayExpressionStash = __async(function () {
	var _foo4 = foo();

	return __await(bar(), function (_bar3) {
		return [1, _foo4, _bar3, baz()];
	});
});

var objectExpressionStash = __async(function () {
	var _foo5 = foo();

	return __await(bar(), function (_bar4) {
		return {
			one: 1,
			array: [1, 2, 3],
			obj: { one: 1 },
			foo: _foo5,
			bar: _bar4,
			baz: baz()
		};
	});
});

var single = __async(function () {
	return __await(foo(), function (_foo6) {
		var f = _foo6;
		return f + 1;
	});
});

var double = __async(function () {
	return __await(foo(), function (_foo7) {
		var f = _foo7;
		return __await(bar(), function (_bar5) {
			var b = _bar5;
			return f && b;
		});
	});
});

var logicalLeft = __async(function () {
	return __await(left, function (_left) {
		return _left && right;
	});
});

var logicalRight = __async(function () {
	var _left2 = left;
	return __await(_left2 ? right : 0, function (_right) {
		return _left2 && _right;
	});
});

var logicalBoth = __async(function () {
	return __await(left, function (_left3) {
		var _ref2 = _left3;
		return __await(_ref2 ? right : 0, function (_right2) {
			return _ref2 && _right2;
		});
	});
});

var binaryLeft = __async(function () {
	return __await(left, function (_left4) {
		return _left4 + right;
	});
});

var binaryRight = __async(function () {
	var _left5 = left;
	return __await(right, function (_right3) {
		return _left5 + _right3;
	});
});

var binaryBoth = __async(function () {
	return __await(left, function (_left6) {
		var _ref3 = _left6;
		return __await(right, function (_right4) {
			return _ref3 + _right4;
		});
	});
});

var binaryAndLogical = __async(function () {
	return __await(left, function (_left7) {
		var _ref5 = _left7;
		return __await(middle, function (_middle) {
			var _ref4 = _ref5 + !_middle;

			return __await(_ref4 ? right : 0, function (_right5) {
				return _ref4 && _right5;
			});
		});
	});
});

var beforeIfBody = __async(function () {
	return __await(bar, function (_bar6) {
		var foo = _bar6;
		if (foo) {
			return 1;
		} else {
			return 0;
		}
	});
});

var insideIfBodySet = __async(function () {
	var result;
	return __await(function () {
		if (foo) {
			return __await(bar, function (_bar7) {
				result = _bar7;
			});
		} else {
			return __await(baz, function (_baz2) {
				result = _baz2;
			});
		}
	}(), function () {
		return result;
	});
});

var insideIfBodyReturn = __async(function () {
	if (foo) {
		return bar;
	} else {
		return baz;
	}
});

var insideIfConditional = __async(function () {
	return __await(foo, function (_foo8) {
		if (_foo8) {
			return 1;
		} else {
			return 0;
		}
	});
});

var ternaryTest = __async(function () {
	return __await(foo, function (_foo9) {
		return _foo9 ? bar : baz;
	});
});

var ternaryConsequent = __async(function () {
	var _foo10 = foo;
	return __await(_foo10 ? bar : 0, function (_bar9) {
		return _foo10 ? _bar9 : baz;
	});
});

var ternaryAlternate = __async(function () {
	var _foo11 = foo;
	return __await(_foo11 ? 0 : baz, function (_baz4) {
		return _foo11 ? bar : _baz4;
	});
});

var ternaryBoth = __async(function () {
	var _foo12 = foo;
	return _foo12 ? bar : baz;
});

var ternaryAll = __async(function () {
	return __await(foo, function (_foo13) {
		var _ref6 = _foo13;
		return _ref6 ? bar : baz;
	});
});

var onlyIf = __async(function () {
	var _exit;

	return __await(function () {
		if (foo) {
			return __await(bar, function (_bar10) {
				var baz = _bar10;
				if (baz) {
					return baz;
				}
			});
		}
	}(), function () {
		if (_exit) return _result;
		return 0;
	});
});

var testThis = __async(function () {
	var _this = this;

	return __await(_this.foo, function (_this$foo) {
		var _ref8 = _this$foo;
		return __await(_this.bar, function (_this$bar) {
			var _ref7 = _ref8 + _this$bar;

			return __await(_this.baz, function (_this$baz) {
				return _ref7 + _this$baz;
			});
		});
	});
});

// Arrow functions
var arrow = __async(function (foo) {
	return foo;
});

var awaitKey = function (key) {
	var _this = this;

	// Test this expressions in arrow functions
	return __async(function () {
		return _this[key];
	});
};

var variables = __async(function (value) {
	var a = 1;
	return __await(value, function (_value) {
		var b = _value,
		    c = 3;
		return a + b + c;
	});
});

var catchAndFallback = __async(function (value) {
	return __try(function () {
		return value;
	}).catch(function (e) {
		return "Error";
	});
});

var catchAndIgnore = __async(function (value) {
	return __await(__try(function () {
		return value;
	}).catch(function (e) {}), __empty);
});

var catchAndAwait = __async(function (value) {
	return __try(foo).catch(function (e) {
		return __await(value, __empty);
	});
});

var catchAndLog = __async(function (value) {
	var result;
	return __await(__try(function () {
		return __await(value, function (_value5) {
			result = _value5;
		});
	}).catch(function (e) {
		result = "Some Error";
	}), function () {
		console.log("result:", result);
		return result;
	});
});

var finallyExample = __async(function (value) {
	// TODO: Make this actually return the result of foo()
	return __finally(__await(__try(foo), __empty), function (_wasThrown, _result2) {
		console.log("finished foo, might rethrow");
		if (_wasThrown) throw _result2;else return _result2;
	});
});

var finallySuppressedExample = __async(function (value) {
	return __finally(__try(test), function () {
		return "Ignored";
	});
});

var awaitAll = __async(function (list) {
	return __forTo(list, function (i) {
		return __await(list[i], __empty);
	});
});

var awaitWithBreak = __async(function (list) {
	var _interrupt;

	var i = 0;return __for(function () {
		return !_interrupt && i < list.length;
	}, function () {
		return i++;
	}, function () {
		return __await(list[i], function (_list$i2) {
			if (_list$i2) {
				_interrupt = 1;
				return;
			}
		});
	});
});

var awaitWithContinue = __async(function (list) {
	return __forTo(list, function (i) {
		return __await(list[i], function (_list$i3) {
			var result = _list$i3;
			if (!result) {
				return;
			}
			console.log(result);
		});
	});
});

var whileLoop = __async(function () {
	let shouldContinue = true;
	return __for(function () {
		return shouldContinue;
	}, void 0, function () {
		return __await(foo, function (_foo16) {
			shouldContinue = _foo16;
			console.log(shouldContinue);
		});
	});
});

var doWhile = __async(function () {
	return __do(function () {
		var _console = console;
		return __await(foo, function (_foo17) {
			_console.log.call(_console, _foo17);
		});
	}, function () {
		return false;
	});
});

var doWhileRet = __async(function () {
	var _exit2;

	return __await(__do(function () {
		return __await(foo, function (_foo18) {
			if (_foo18) {
				return _exit2 = 1, true;
			}
		});
	}, function () {
		return !_exit2;
	}), function (_result4) {
		if (_exit2) return _result4;
	});
});

var awaitPredicate = __async(function () {
	var _exit3;

	return __await(__for(function () {
		var _ref9 = !_exit3;

		return __await(_ref9 ? foo() : 0, function (_foo19) {
			return _ref9 && _foo19;
		});
	}, void 0, function () {
		console.log("Got a foo!");
		if (1) {
			return _exit3 = 1, true;
		}
	}), function (_result5) {
		if (_exit3) return _result5;

		return false;
	});
});

var awaitPredicateBinary = __async(function () {
	return __await(__for(function () {
		return __await(foo(), function (_foo20) {
			var _ref10 = _foo20;
			return __await(_ref10 ? bar() : 0, function (_bar11) {
				return _ref10 && _bar11;
			});
		});
	}, void 0, function () {
		console.log("hi");
	}), __empty);
});

var awaitPredicateConditional = __async(function () {
	return __await(__for(function () {
		return __await(foo(), function (_foo21) {
			var _ref11 = _foo21;
			return _ref11 ? bar() : baz();
		});
	}, void 0, function () {
		console.log("hi");
	}), __empty);
});

var forInAwaitValue = __async(function () {
	return __await(foo, function (_foo22) {
		for (var key in _foo22) {
			console.log(key);
		}
	});
});

var forInAwaitBody = __async(function () {
	return __forIn(foo, function (key) {
		return __await(foo[key], __empty);
	});
});

var forInAwaitWithReturn = __async(function () {
	var _exit4;

	return __await(__forIn(foo, function (key) {
		return __await(foo[key], function (_foo$key2) {
			if (_foo$key2) {
				return _exit4 = 1, true;
			}
		});
	}, function () {
		return _exit4;
	}), function (_result6) {
		if (_exit4) return _result6;

		return false;
	});
});

var awaitDiscriminant = __async(function () {
	return __await(foo, function (_foo23) {
		switch (_foo23) {
			case 1:
				console.log("1");
				break;
			case 2:
				console.log("2");
				break;
			default:
				console.log("some other thing");
				break;
		}
	});
});

var awaitBody = __async(function () {
	switch (foo) {
		case false:
		case true:
			return foo;
		default:
			return bar;

	}
});

var awaitBodyIndirect = __async(function () {
	switch (foo) {
		case false:
		case true:
			return foo;
		default:
			return __await(bar(), function (_bar13) {
				return;
			});
	}
});

var awaitCase = __async(function () {
	return __switch(foo, [[function () {
		return true;
	}, function () {
		return "foo";
	}], [function () {
		return bar;
	}, function () {
		return "foobar";
	}], [void 0, function () {
		return "not fubar";
	}]]);
});

var awaitNothing = __async(function () {
	return __await(foo, function (_foo24) {
		switch (_foo24) {
			case true:
				return "foo";
			case bar:
				return "foobar";
			default:
				return "not fubar";
		}
	});
});

var awaitComplex = __async(function (foo, bar) {
	var _break;

	return __switch(foo, [[], [function () {
		return 0;
	}], [function () {
		return 1;
	}, function () {
		return false;
	}], [function () {
		return bar;
	}, function () {
		if (foo) {
			_break = 1;
			return;
		}
		if (foo === false) {
			return false;
		}
	}, function () {
		return _break;
	}], [function () {
		return 2;
	}, function () {
		console.log("implicit break");
		return true;
	}]]);
});
